@@ -1,34 +1,31 @@
-from . import color
+from . color import Color
+from functools import cached_property
+from typing import Iterator, Optional, Tuple, Union
 import importlib
 import re
 import string
 
 _ALLOWED = set(string.ascii_letters + string.digits)
 
+ColorKey = Union[int, slice, str, Color, Tuple[int, int, int]]
+
 
 class Colors:
-    """DOX HERE"""
+    """A collection of named colors that acts like an array, a dictionary
+    and a namespace.
+
+    `Colors` is indexed by a type `ColorKey`, which may be an
+    `int`, `slice`, `str`, `Color`, or`Tuple[int, int, int]]`.
+    """
 
     def __init__(self, *palettes, canonicalize_gray='gray', default='black'):
+        from . import color
+
         class Color(color.Color):
             COLORS = self
 
-        super().__setattr__('Color', Color)
-
-        gt = self._canonicalize_gray = canonicalize_gray
-        if not gt:
-            self._replacements = ()
-        else:
-            gt = 'gray' if gt is True else gt.lower()
-            gf = 'grey' if gt == 'gray' else 'gray'
-            if gt not in ('gray', 'grey'):
-                raise ValueError('Don\'t understand canonicalize_gray=%s' % gt)
-
-            self._replacements = (
-                (re.compile(r'\b%s\b' % gf).sub, gt),
-                (re.compile(r'\b%s\b' % gf.capitalize()).sub, gt.capitalize()),
-            )
-
+        self.__dict__['Color'] = Color
+        self._canonicalize_gray = canonicalize_gray
         self._name_to_rgb = {}
         self._rgb_to_name = {}
         self._palettes = [self._add_palette(s) for s in palettes]
@@ -38,22 +35,26 @@ class Colors:
         }
         self._default = self.get(str(default)) or next(iter(self._rgb_to_name))
 
-    def get(self, key, default=None):
+    def get(self, key: ColorKey, default: Optional[Color] = None):
+        """Return the value for `key` if it exists, else `default`."""
         try:
             return self[key]
         except KeyError:
             return default
 
-    def items(self):
-        return self._name_to_rgb.items()
+    def items(self) -> Iterator[Tuple[str, Color]]:
+        """Return an iterator of name, Color pairs. Some colors might repeat"""
+        return self._colors.items()
 
-    def values(self):
-        return self._name_to_rgb.values()
+    def values(self) -> Iterator[Color]:
+        """Return an iterator of Colors. Some colors might repeat"""
+        return self._colors.values()
 
-    def keys(self):
-        return self._name_to_rgb.keys()
+    def keys(self) -> Iterator[str]:
+        """Return an iterator of strings, unique color names"""
+        return self._colors.keys()
 
-    def closest(self, color):
+    def closest(self, color: Color) -> Color:
         """
         Return the closest named color to `color`.  This can be quite slow,
         particularly if there are many colors.
@@ -67,8 +68,14 @@ class Colors:
     def __call__(self, *args, **kwds):
         return self.Color(*args, **kwds)
 
-    def __getitem__(self, name):
+    def __getitem__(self, name: ColorKey) -> Color:
         """Try to convert string item into a color"""
+        if isinstance(name, (int, slice)):
+            return self._color_list[name]
+
+        if isinstance(name, tuple):
+            return self.Color(*name)
+
         canonical = self._canonical_name(name)
         try:
             return self._canonical_to_rgb[canonical]
@@ -79,11 +86,11 @@ class Colors:
     def __setitem__(self, name, rgb):
         raise KeyError(name)
 
-    def __contains__(self, x):
-        """Return true if this string name appears in the table canonically"""
+    def __contains__(self, x: ColorKey) -> bool:
+        """Return true if this ColorKey appears in the table canonically"""
         return self._canonical_name(x) in self._canonical_to_rgb
 
-    def __getattr__(self, name):
+    def __getattr__(self, name: str) -> Color:
         if name.startswith('_'):
             return super().__getattribute__(name)
         try:
@@ -96,16 +103,16 @@ class Colors:
             return super().__setattr__(name, value)
         raise AttributeError(name)
 
-    def __len__(self):
-        return len(self._name_to_rgb)
+    def __len__(self) -> int:
+        return len(self._color_list)
 
-    def __iter__(self):
-        return iter(self._name_to_rgb)
+    def __iter__(self) -> Iterator[Color]:
+        return iter(self._color_list)
 
-    def __eq__(self, x):
+    def __eq__(self, x) -> bool:
         return __class__ == x.__class__ and self._name_to_rgb == x._name_to_rgb
 
-    def __ne__(self, x):
+    def __ne__(self, x) -> bool:
         return not (self == x)
 
     def __add__(self, x):
@@ -168,6 +175,28 @@ class Colors:
             name = name.replace('grey', 'gray')
         return ''.join(i for i in name if i in _ALLOWED)
 
+    @cached_property
+    def _colors(self):
+        return {k: self.Color(*v) for k, v in self._name_to_rgb.items()}
+
+    @cached_property
+    def _color_list(self):
+        return list(self._colors.values())
+
+    @cached_property
+    def _replacements(self):
+        if not (gt := self._canonicalize_gray):
+            return ()
+
+        gt = 'gray' if gt is True else gt.lower()
+        gf = 'grey' if gt == 'gray' else 'gray'
+        if gt not in ('gray', 'grey'):
+            raise ValueError('Don\'t understand canonicalize_gray=%s' % gt)
+
+        regular = re.compile(r'\b%s\b' % gf).sub, gt
+        upper = re.compile(r'\b%s\b' % gf.capitalize()).sub, gt.capitalize()
+        return regular, upper
+
 
 """Some colors have multiple names; a best name needs to be chosen.
    palette.PRIMARY_NAMES is a list of names to use by preference.
